var Structr = require('structr'),
RemoteModel = require('./remote'),
Collection = require('./collection'),
vine = require('vine'),
concrete = require('./concrete');



module.exports = RemoteModel.extend(concrete.partial, {
	

	/**
	 * id does not exist? it's new.
	 */

	'isNew': function()
	{
		return this.doc._id == undefined;
	},

	/**
	 */

	'collection': function(collectionName, model, search)
	{
		//root, or relative
		var channel = collectionName.substr(0,1) == '/' ? collectionName : this._channel() + '/' + collectionName;

		return this['__' + collectionName] || (this['__' + collectionName] = this.newModel('Collection', { channel: channel, Model: model, parent: this, doc: search }));
	},

	/**
	 */

	'save': function(doc, callback)
	{
		if(typeof doc == 'function')
		{
			callback = doc;
			doc = {};
		}

		if(!doc) doc = {};

		if(this.parent) doc.parent = this.parent.get('_id');

		this._set(doc);

		var isNew = this.isNew(), self = this;


		function onSave()
		{
			if(callback) callback();

			if(isNew && self._collection) self._collection._add(self);
		}

		this.load( isNew ? 'POST' : 'PUT', this.doc, onSave);
		return this;
	},

	/**
	 */
	
	'remove': function(onRemove)
	{
		this._pull('DELETE', this.removeData(), onRemove);


		//remove from the collection if it exists.
		if(this._collection) this._collection._remove(this.doc);

		this.change('remove');

		return this;
	},

	/**
	 */

	'removeData': function()
	{
		return null;
	},

	/**
	 */

	'override _load': function(method, data, callback)
	{
		if(!data) data = {};

		if(this.isNew()) data = this.doc;


		return this._super(method, data, callback);
	},


	/**
	 */

	'_channel': function()
	{
		if(this.isNew()) return this.channel;
		return this.channel + '/' + this.doc._id;
	},

	/**
	 */

	'_onResponse': function(response)
	{
		if(response.result)
		{
			this._set(response.result instanceof Array ? response.result[0] : response.result);
		}

		if(this.router)
		{
			if(response.errors)
			{
				this.router.push('alert/error', response.errors);
			}

			if(response.warnings)
			{
				this.router.push('alert/warning', response.warnings);
			}
		}
	},

	/**
	 */

	'override _set': function(doc)
	{
		this._super(doc);

		// this.loaded = this.doc._id != undefined;
	},

	/**
	 */

	'static find': function(query, callback)
	{
		if(typeof query == 'function')
		{
			callback = query;
			query = {};
		}

		var channel = this.prototype.channel;

		if(query._id)
		{
			channel += '/' + query._id;
			delete query._id;
		}

		var model = this.prototype.model('Collection');

		var col = new model( { channel: channel, Model: this });

		col.load(query).subscribeOnce('reset', function(e)
		{
			if(callback) callback(col);
		});

		return col;
	},

	/**
	 */

	'static findOne': function(query, callback)
	{
		this.find(query, function(collection)
		{
			callback(collection.item(0));
		});
	},

	/**
	 */

	'toJSON': function()
	{
		return this.doc;
	}
});




