var Structr = require('structr');


module.exports = Structr({
	
	/**
	 */

	'__construct': function(target, modifiers)
	{
		this._defs = {};
		this._modifiers = modifiers;

		for(var property in target)
		{
			this.add(property, target[property]);
		}	
	},

	/**
	 */


	'add': function(property, definition)
	{
		this._defs[property] = new exports.Definition(property, definition, this._modifiers);
	},

	/**
	 */

	'definition': function(property)
	{
		return this._defs[property];
	},

	/**
	 */

	'apply': function(model)
	{
		for(var prop in this._defs)
		{
			this._defs[prop].apply(model);
		}	
	},

	/**
	 */

	'set': function(model, property, value)
	{
		var def = this.definition(property);

		return def ? def.set(model, value) : value;
	}
});


exports.Definition = Structr({
	
	/**
	 */

	'__construct': function(property, def, modifiers)
	{
		this.property = property;
		this._modifiers = modifiers;
		this._def = def;

		for(var prop in def) this[prop] = def[prop];
	},

	/**
	 */

	'apply': function(model)
	{
		for(var modifier in this._def)
		{
			this._modifiers[modifier].apply(model, this);
		}
	},

	/**
	 */

	
	'set': function(model, value)
	{
		var newValue = value;

		for(var modifier in this._def)
		{
			var mod = this._modifiers[modifier];

			if(mod)
			{
				newValue = mod.set(model, newValue, this);
			}
		}

		return newValue;
	}
})
