var beanpole = require('beanpole/lib/core'),
hookBean = require('beanpole/lib/core/beans/hook.core'),
// hookHttpBean = require('beanpole/node/beans/hook.http.mesh'),
socketBean = require('beanpole/lib/web/beans/hook.socket.io.client'),
models = require('./core/models'),
Schema = require('./core/schema'),
modifiers = require('./core/modifiers');

models.Item = require('./core/models/item');
models.Collection = require('./core/models/collection');
models.Model = require('./core/models/concrete').Model;


exports.models = models;
exports.modifiers = modifiers;



exports._initModel = function(Model, name, appModels, router)
{

	var AppModel = Model.extend({ router: router, 


		//used for serialization
		name: Model.prototype.name || name,

		//schema for the model
		_schema: new Schema(Model.schema, modifiers),


		/**
		 */

		'newModel': function(type, ops)
		{
			var clazz = this.model(type);

			return new clazz(ops);
		},

		/**
		 */

		'model': function(type)
		{
			var clazz;

			if(this.models)
			{
				clazz = appModels[this.models[type]];
			}
			
			return clazz ? clazz : appModels[type || this.channel];
		}
	});


	appModels[name] = appModels[AppModel.prototype.name] /*= appModels[Model.prototype.channel]*/ = AppModel;
}



/**
 * initializes the models on the given router. Must already be connected
 */

exports.init = function(router)
{
	var hostModels = {};

	for(var name in models)
	{
		var Model = models[name];

		if(!Model.extend) continue;

		exports._initModel(Model, name, hostModels , router);
	}


	return router.models = hostModels;
}


/**
 * connects malt to a server
 */

exports.connect = function(host, onConnect)
{
	var router = beanpole.router();


	exports.init(router);


	//hook core 
	hookBean.plugin(router);

	//hook via websockets
	socketBean.plugin(router);

	if(onConnect) router.on('push -one -public spice.io/ready', function()
	{
		onConnect(router);
	})

	router.push('init');

	return router;
}

